/*
 *	String gadget "editor"
 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>
#include <string.h>
#include <unistd.h>

#include "hr_defines.h"
#include "hr_colors.h"
#include "hr_fonts.h"
#include "hr_mouse.h"
#include "hr_general.h"
#include "hr_keyboard.h"
#include "hr_buttons.h"
#include "hr_bobs.h"
#include "hr_bitmaps.h"
#include "hr_input_handler.h"
#include "hr_zones.h"
#include "hr_strings.h"


/* */
#define CHAR_ENTER     10
#define CHAR_ESCAPE    27
#define CHAR_CURSLEFT  68
#define CHAR_CURSRIGHT 67
#define CHAR_DELETE    4
#define CHAR_BACKSPACE 127


/* Updates button gfx. Note that text in buttons is "unfolded" so
 * that control sequences are visible.*/
/* FIX!!!!!!!!!!!! */
void hr_editor_button_update(struct hr_button *b,unsigned char *text,
									  int offset,int loc)
{
	int printchars;
	/**/
	printchars=MIN(strlen(text),
						((b->zone->x2)-(b->zone->x1)-1)/(b->font->width));
	hr_clear_button(b);
	/* Draw string.*/
	hr_write(((b->zone->x1)+(b->zone->x2)-printchars*(b->font->width))/2,
				((b->zone->y1)+(b->zone->y2)-(b->font->height))/2+1,
				hr_get_color_index(b->colors,COLOR_FOREGROUND),
				hr_get_color_index(b->colors,COLOR_BACKGROUND),				
				text,
				b->font,
				offset,printchars,
				FONT_UNFORMATTED);
	/* Draw cursor.*/
	hr_write((((b->zone->x1)+(b->zone->x2)-printchars*(b->font->width))/2+
				 (loc-offset)*(b->font->width)),
				((b->zone->y1)+(b->zone->y2)-(b->font->height))/2+1,
				hr_get_color_index(b->colors,COLOR_FOREGROUND),
				hr_get_color_index(b->colors,COLOR_BACKGROUND),				
				(loc<strlen(text))?(text):((unsigned char *)" "),
				b->font,
				(loc<strlen(text))?(loc):(0),1,
				FONT_UNFORMATTED|FONT_REVERSE|FONT_SINGLECOLBGR);
}


/* Lets one edit a string within a button interactively.
 * Very useful for making text-entry gadgets.
 * It looks better if text-entries are within an Amiga-style
 * double-bevelbox, that is, a raised bevelbox that contains a
 * pressed "button" with the text in it.*/
struct hr_button *hr_string_edit(struct hr_button *b,
											int (*func)(),int flags)
{
	struct hr_button *newb;
	int loc,offset,l,visible_characters;
	int another_one,enter_received=0;
	unsigned char tmp[HR_BUFFER_SIZE];
	/* Do nothing if b points to NULL.*/
	newb=b;
	if(b!=NULL){
		do {
			/* Calc the amount of visible characters,
			 * and only loop if at least one character fits on screen.*/
			/* FIX! */
			visible_characters=(((b->zone->x2)-(b->zone->x1)+1)/
									  (b->font->width)-1);
			if(visible_characters>0) {
				/* Copy the string to be edited to tmp[] buffer.*/
				strncpy(tmp,
						  ((b->flags & BUTTON_PRESSED)?(b->text2):(b->text)),
						  HR_BUFFER_SIZE-2);
				/* Just to make sure keyboard.lastkey is 0.*/
				hr_clearkey();
				/* Put cursor at the end of the string.*/
				loc=strlen(tmp);
				/* Calc initial offset */
				offset=0;
				if(strlen(tmp)>=visible_characters)
				  offset=strlen(tmp)-visible_characters-1;
				/* Redraw button gfx.*/
				hr_editor_button_update(b,tmp,offset,loc);
				/* Make sure no unnecessary repositioning is done.*/
				hr_input_handler(NULL,func,AMB_RELEASED);
				
				/* Loop while linefeed not received, AND
				 * left button not clicked outside this button. */
				enter_received=0;
				newb=b;
				while((!(enter_received) &&
						 (newb=hr_input_handler(b,func,
														KBD_PRESSED|LMB_PRESSED|
														NO_EDITING|IGNORE_HOTKEYS)) &&
						 (newb==b)))
				  {
#if 0
					  
					  KORjAAAaaaaaaaaaaaaaaaaaaaaa FIX FIX FIX;
					  
					  /* Mouse event (LMB was clicked on the string gadget b).*/
					  if(HR_LMB){
						  /* Calc new cursor location.*/
						  xx=((b->zone->x1)+(b->zone->x2))/2-
							 (MIN(strlen(tmp),visible_characters)*
							  (b->font->width))/2;
						  if(hr_mouse.x<xx)
							 nloc=0;
						  else
							 nloc=(hr_mouse.x-xx)/(b->font->width);
						  /* Update loc and redraw button gfx
							* if location was changed.*/
						  if(nloc<0)
							 nloc=0;
						  if(nloc>strlen(tmp))
							 nloc=strlen(tmp);
						  if((nloc!=loc) && (nloc<=visible_characters-1)){
							  loc=nloc;
							  /* Redraw button gfx.*/
							  hr_editor_button_update(b,tmp,offset,loc);
						  }
						  hr_input_handler(NULL,func,AMB_RELEASED);
					  }
					  else
#endif
						 /* Keyboard event.*/
						 if(hr_keyboard.lastkey!=0){
							 switch(hr_keyboard.lastkey) {
							  case CHAR_ENTER:
								 /* Awayyyyyyyyy.*/
								 enter_received=1;
								 break;
							  case CHAR_ESCAPE:
								 /* Escape sequences. */
								 if(hr_getkey()==91){
									 switch(hr_getkey()) {
									  case CHAR_CURSLEFT:
										 /* Curs left */
										 if(loc>0)
											--loc;
										 else if(offset>0)
											--offset;
										 break;
									  case CHAR_CURSRIGHT:
										 /* Curs right */
										 if(loc<strlen(tmp)){
											 ++loc;
											 if((loc+offset)>=visible_characters)
												++offset;
										 }
										 break;
									 }
								 }
								 break;
							  case CHAR_DELETE:
								 /* Delete (Ctrl-D)*/
								 if(loc<strlen(tmp)) {
									 for(l=loc;l<strlen(tmp);l++)
										tmp[l]=tmp[l+1];
									 /* Move offset towards beginning if
									  * the string is longer than fits in the gadget
									  * AND if we are near the end of text.*/
									 if((offset>0)&&
										 ((strlen(tmp)-offset-loc)<visible_characters)){
										 --offset;
										 ++loc;
									 }
								 }
								 break;
							  case CHAR_BACKSPACE:
								 /* Backspace */
								 if(loc>0){
									 if((offset>0) &&
										 (strlen(tmp)>=visible_characters))
										--offset;
									 --loc;
									 for(l=loc;l<strlen(tmp);l++)
										tmp[l]=tmp[l+1];
								 }
								 break;
							  default:
								 /* Insert a character in buffer[], if it fits.*/
								 if(strlen(tmp)<HR_BUFFER_SIZE-2) {
									 /* Make space for a new char */
									 for(l=strlen(tmp);l>=(loc);l--)
										tmp[l+1]=tmp[l];
									 /* Insert the char */
									 tmp[loc]=hr_keyboard.lastkey;
									 ++loc;
#if 0
									 else
										++offset;
#endif
								 }
								 break;
							 }
							 /* Redraw button gfx.*/
							 hr_editor_button_update(b,tmp,offset,loc);
						 }
				  }
				/* Update button */
				hr_update_button(b,
									  (b->flags & BUTTON_PRESSED)?
									  (UPDATE_TEXT2):(UPDATE_TEXT),
									  tmp);
				/* Set flag.*/
				b->flags |= BUTTON_STRING_EDITED;
				/* Not really necessary...*/
				hr_clearkey();
			}
			/* Edit another string, if the used clicked in another
			 * string gadget. Kinda kludgish, but should work fine,
			 * and does not use recursion :-) */
			another_one=0;
			if((newb!=NULL)&&(!(enter_received))){
				if(newb->flags & BUTTON_ACCEPTS_INPUT){
					b=newb;
					another_one=1;
				}
			}
		}while(another_one);
	}
	return newb;
}
